
##LibName#Regression
##LibDescription#Funktionen zur Bestimmung und Darstellung von:
##LibDescription# - Regressionskoeffizienten  
##LibDescription# - Prognoseintervallen 
##LibDescription# - Konfidenzintervallen fr Regressionsgerade
##LibDescription# - Regressionsgerade
##LibCopyright# Rainer Schlittgen  

Regress<-function(y,x,stdfehler=FALSE){ 
##FunctionName#Regress
##FunctionDescription# Bestimmung der Regressionskoeffizienten y=beta0 + beta1*x1 +...+ betap*xp + U
##FunctionDescription#  optional knnen auch die Standardfehler bestimmt werden mittels des
##FunctionDescription#  zustzliches Argumentes: , stdfehler=TRUE 
##FunctionSyntax#Regress(y,x)
##FunctionDescription# 

 ab <- lm(y~x)
 beta <- coefficients(ab)
 b <- cbind(beta)
 if (stdfehler)
  { 
    xx <- summary(ab)$cov.unscaled 
    sigma <- summary(ab)$sigma  
    stderror <- sqrt(diag(xx))*sigma
    b<-cbind(beta,stderror) 
  } 
 b
 }
 
Progreg<-function(y,x,x0,gamma=0,typ="none"){ 
##FunctionName#Progreg
##FunctionDescription# Bestimmung von Prognosewerten mit linearer Regression an den in x0 angegebenen Stellen
##FunctionDescription#  Die Punktprognosen knnen ergnzt werden durch Konfidenz- oder Prognoseintervalle 
##FunctionDescription#  Dazu sind die optionalen Angaben ,gamma=0.95, typ="konfidenz" bzw. ,gamma=0.95, typ="prognose" zu ergnzen.
##FunctionDescription#  Fr gamma kann natrlich auch ein anderes Niveau gewhlt werden.
##FunctionSyntax#Progreg(x,y,x0)
##FunctionDescription# 

  
 ab <- lm(y~x)
 beta <- coefficients(ab)
 beta <- matrix(beta,length(beta),1)
 if (is.matrix(x0))  
   { 
     x1 <- matrix(rep(1,(ncol(x0)+1)*length(x0)),length(x0),(ncol(x0)+1)) 
     x1[,2:ncol(x1)] <- x0
     x0 <- x1
   } 
 if (is.vector(x0)) 
   { 
     x1 <- matrix(rep(1,2*length(x0)),length(x0),2) 
     x1 <- x1[,2]<-x0
     x0 <- matrix(c(rep(1,length(x0)),x0),length(x0),2)
   }    
 pred <- x0%*%beta  
 intervall <- pred  
 if (typ=="konfidenz")
   { 
    p <- nrow(beta)
    n <- length(y) 
    ta <- qt(1-(1-gamma)/2,n-p)
    g <- rep(0,nrow(x0))
    xx <- summary(ab)$cov.unscaled
    sigma <- summary(ab)$sigma
    x1<-t(x0) 
    for (i in 1:nrow(x0)) { g[i] <- ta*sqrt(x0[i,]%*%xx%*%x1[,i])*sigma } 
    lowlim <- intervall-g
    uplim <- intervall+g    
    intervall <- cbind(pred,lowlim,uplim) 
   }
 if (typ=="prognose")
   { 
    p <- nrow(beta)
    n <- length(y) 
    ta <- qt(1-(1-gamma)/2,n-p)
    g <- rep(0,nrow(x0))
    xx <- summary(ab)$cov.unscaled 
    sigma <- summary(ab)$sigma
    x1<-t(x0) 
    for (i in 1:nrow(x0)) { g[i] <- ta*sqrt(1+x0[i,]%*%xx%*%x1[,i])*sigma }
    lowlim <- intervall-g
    uplim <- intervall+g
    intervall <- cbind(pred,lowlim,uplim) 
   } 
  intervall
 } 
 
 
 
Plotreg<-function(y,x,x0,gamma=0,typ="none"){ 
##FunctionName#Plotreg
##FunctionDescription# Darstellung der Regressionsgeraden bei einer einfachen linearen Regression mit Konfidenz- oder Prognoseintervallen an den in x0 angegebenen Stellen
##FunctionDescription#  Dazu sind die optionalen Angaben ,gamma=0.95, typ="konfidenz" bzw. ,gamma=0.95, typ="prognose" zu ergnzen.
##FunctionDescription#  Fr gamma kann natrlich auch ein anderes Niveau gewhlt werden.
##FunctionSyntax#Plot.reg(x,y,x0)
##FunctionDescription# 
  

 if (is.matrix(x0)) {stop("Nur fr einfache lineare Regression (x0 muss Vektor sein.)")} 
 ab <- lm(y~x) 
 
 if (typ=="none")
   {
    plot(x,y,type="p",pch=16,cex=1.5, col="blue",main="Regressionsgerade") 
   abline(ab) 
   
   } 
 if (typ=="konfidenz")
  {
   intervall <- Progreg(y,x,x0,gamma,typ="konfidenz")
   xl<-c(min(c(x,x0)),max(c(x,x0)))
   yl<-c(min(c(y,intervall[,1],intervall[,2])),max(c(y,intervall[,1],intervall[,3])))
   plot(x,y,type="p",xlim=xl,ylim=yl,pch=16,cex=1.5, col="blue",main="Regressionsgerade mit Konfidenzintervallen") 
   abline(ab) 
   for (i in 1:nrow(intervall)) 
     { 
       lines(rep(x0[i],3),intervall[i,],type="b",pch="-",cex=2,col="darkred")
     }
  } 
 if (typ=="prognose")
  {
   intervall <- Progreg(y,x,x0,gamma,typ="prognose")
   xl<-c(min(c(x,x0)),max(c(x,x0)))
   yl<-c(min(c(y,intervall[,1],intervall[,2])),max(c(y,intervall[,1],intervall[,3])))
   plot(x,y,type="p",xlim=xl,ylim=yl,pch=16,cex=1.5, col="blue",main="Regressionsgerade mit Prognoseintervallen") 
   abline(ab) 
   for (i in 1:nrow(intervall)) 
     { 
       lines(rep(x0[i],3),intervall[i,],type="b",pch="-",cex=2,col="darkred")
     }
  }   
 }